@@ -1527,7 +1527,6 @@ OvsUpdateAddressAndPort(OvsForwardingContext *ovsFwdCtx,
 
     csumInfo.Value = NET_BUFFER_LIST_INFO(ovsFwdCtx->curNbl,
                                           TcpIpChecksumNetBufferListInfo);
-
     /*
      * Adjust the IP header inline as dictated by the action, and also update
      * the IP and the TCP checksum for the data modified.
@@ -1536,7 +1535,6 @@ OvsUpdateAddressAndPort(OvsForwardingContext *ovsFwdCtx,
      * ChecksumUpdate32(). Ignoring this for now, since for the most common
      * case, we only update the TTL.
      */
-     /*Only tx direction the checksum value will be reset to be PseudoChecksum*/
 
     if (isSource) {
         addrField = &ipHdr->saddr;
@@ -1553,7 +1551,7 @@ OvsUpdateAddressAndPort(OvsForwardingContext *ovsFwdCtx,
                         ((BOOLEAN)csumInfo.Receive.UdpChecksumSucceeded ||
                          (BOOLEAN)csumInfo.Receive.UdpChecksumFailed);
         }
-        if (isTx && l4Offload) {
+        if (l4Offload) {
             *checkField = IPPseudoChecksum(&newAddr, &ipHdr->daddr,
                 tcpHdr ? IPPROTO_TCP : IPPROTO_UDP,
                 ntohs(ipHdr->tot_len) - ipHdr->ihl * 4);
@@ -1574,7 +1572,7 @@ OvsUpdateAddressAndPort(OvsForwardingContext *ovsFwdCtx,
                          (BOOLEAN)csumInfo.Receive.UdpChecksumFailed);
         }
 
-       if (isTx && l4Offload) {
+       if (l4Offload) {
             *checkField = IPPseudoChecksum(&ipHdr->saddr, &newAddr,
                 tcpHdr ? IPPROTO_TCP : IPPROTO_UDP,
                 ntohs(ipHdr->tot_len) - ipHdr->ihl * 4);
@@ -1583,7 +1581,7 @@ OvsUpdateAddressAndPort(OvsForwardingContext *ovsFwdCtx,
 
     if (*addrField != newAddr) {
         UINT32 oldAddr = *addrField;
-        if ((checkField && *checkField != 0) && (!l4Offload || !isTx)) {
+        if (checkField && *checkField != 0 && !l4Offload) {
             /* Recompute total checksum. */
             *checkField = ChecksumUpdate32(*checkField, oldAddr,
                                             newAddr);
@@ -1592,12 +1590,11 @@ OvsUpdateAddressAndPort(OvsForwardingContext *ovsFwdCtx,
             ipHdr->check = ChecksumUpdate32(ipHdr->check, oldAddr,
                                             newAddr);
         }
-
         *addrField = newAddr;
     }
 
     if (portField && *portField != newPort) {
-        if ((checkField) && (!l4Offload || !isTx)) {
+        if (checkField && !l4Offload) {
             /* Recompute total checksum. */
             *checkField = ChecksumUpdate16(*checkField, *portField,
                                            newPort);
@@ -1712,15 +1709,6 @@ OvsUpdateIPv4Header(OvsForwardingContext *ovsFwdCtx,
         ipHdr->ttl = ipAttr->ipv4_ttl;
         key->ipKey.nwTtl = ipAttr->ipv4_ttl;
     }
-    if (ipHdr->dscp != (ipAttr->ipv4_tos & 0xfc)) {
-        /* ECN + DSCP */
-        UINT8 newTos = (ipHdr->tos & 0x3) | (ipAttr->ipv4_tos & 0xfc);
-        if (ipHdr->check != 0) {
-            ipHdr->check = ChecksumUpdate16(ipHdr->check, ipHdr->tos, newTos);
-        }
-        ipHdr->tos = newTos;
-        key->ipKey.nwTos = newTos;
-    }
 
     return NDIS_STATUS_SUCCESS;
 }
@@ -1869,10 +1857,12 @@ OvsOutputUserspaceAction(OvsForwardingContext *ovsFwdCtx,
 {
     NTSTATUS status = NDIS_STATUS_SUCCESS;
     PNL_ATTR userdataAttr;
-    PNL_ATTR queueAttr;
+    PNL_ATTR egrTunAttr = NULL;
     POVS_PACKET_QUEUE_ELEM elem;
     POVS_PACKET_HDR_INFO layers = &ovsFwdCtx->layers;
     BOOLEAN isRecv = FALSE;
+    OVS_FWD_INFO fwdInfo;
+    OvsIPv4TunnelKey tunKey;
 
     POVS_VPORT_ENTRY vport = OvsFindVportByPortNo(ovsFwdCtx->switchContext,
                                                   ovsFwdCtx->srcVportNo);
@@ -1884,13 +1874,29 @@ OvsOutputUserspaceAction(OvsForwardingContext *ovsFwdCtx,
         }
     }
 
-    queueAttr = NlAttrFindNested(attr, OVS_USERSPACE_ATTR_PID);
     userdataAttr = NlAttrFindNested(attr, OVS_USERSPACE_ATTR_USERDATA);
+    /* Indicate the packet is from egress-tunnel direction */
+    egrTunAttr = NlAttrFindNested(attr, OVS_USERSPACE_ATTR_EGRESS_TUN_PORT);
+
+    /* Fill tunnel key to export to usersspace to calculate the template id */
+    if (egrTunAttr) {
+        RtlZeroMemory(&tunKey,  sizeof tunKey);
+        RtlCopyMemory(&tunKey, &ovsFwdCtx->tunKey, sizeof tunKey);
+        if (!tunKey.src) {
+            status = OvsLookupIPFwdInfo(tunKey.src, tunKey.dst, &fwdInfo);
+            if (status == NDIS_STATUS_SUCCESS && tunKey.dst == fwdInfo.dstIpAddr) {
+                tunKey.src = fwdInfo.srcIpAddr;
+            }
+        }
+        tunKey.flow_hash = tunKey.flow_hash ? tunKey.flow_hash : MAXINT16;
+    }
 
     elem = OvsCreateQueueNlPacket(NlAttrData(userdataAttr),
                                   NlAttrGetSize(userdataAttr),
                                   OVS_PACKET_CMD_ACTION,
-                                  vport, key, ovsFwdCtx->curNbl,
+                                  vport, key,
+                                  egrTunAttr ? &(tunKey) : NULL,
+                                  ovsFwdCtx->curNbl,
                                   NET_BUFFER_LIST_FIRST_NB(ovsFwdCtx->curNbl),
                                   isRecv,
                                   layers);
@@ -2357,7 +2363,7 @@ OvsActionsExecute(POVS_SWITCH_CONTEXT switchContext,
 
     if (status == STATUS_SUCCESS) {
         status = OvsProcessDeferredActions(switchContext, completionList,
-                                           portNo, sendFlags);
+                                           portNo, sendFlags, NULL);
     }
 
     return status;
